import { act, renderHook, waitFor } from '@testing-library/react';

import { SupportedMimeTypes } from '@proton/shared/lib/drive/constants';
import { getItem, removeItem, setItem } from '@proton/shared/lib/helpers/storage';

import { DecryptedLink, useLinksActions, useLinksListing } from '../_links';
import useSharesState from '../_shares/useSharesState';
import { usePhotos } from './PhotosProvider';
import { usePhotosRecovery } from './usePhotosRecovery';

function generateDecryptedLink(linkId = 'linkId'): DecryptedLink {
    return {
        encryptedName: 'name',
        name: 'name',
        linkId,
        createTime: 323212,
        digests: { sha1: '' },
        fileModifyTime: 323212,
        parentLinkId: 'parentLinkId',
        isFile: true,
        mimeType: SupportedMimeTypes.jpg,
        hash: 'hash',
        size: 233,
        metaDataModifyTime: 323212,
        trashed: 0,
        hasThumbnail: false,
        isShared: false,
        rootShareId: 'rootShareId',
    };
}

jest.mock('../_links', () => {
    const useLinksActions = jest.fn();
    const useLinksListing = jest.fn();
    return { useLinksActions, useLinksListing };
});

jest.mock('../_shares/useSharesState');

jest.mock('./PhotosProvider', () => {
    return {
        usePhotos: jest.fn(),
    };
});

jest.mock('../_utils', () => ({
    waitFor: jest.fn().mockImplementation(async (callback) => {
        callback();
    }),
}));

jest.mock('@proton/shared/lib/helpers/storage', () => ({
    getItem: jest.fn(),
    removeItem: jest.fn(),
    setItem: jest.fn(),
}));

jest.mock('../../utils/errorHandling');

const mockedRemoveItem = jest.mocked(removeItem);
const mockedGetItem = jest.mocked(getItem);
const mockedSetItem = jest.mocked(setItem);

describe('usePhotosRecovery', () => {
    const links = [generateDecryptedLink('linkId1'), generateDecryptedLink('linkId2')];
    const mockedUsePhotos = jest.mocked(usePhotos);
    const mockedUseLinksListing = jest.mocked(useLinksListing);
    const mockedUseLinksActions = jest.mocked(useLinksActions);
    const mockedUseShareState = jest.mocked(useSharesState);
    const mockedGetCachedChildren = jest.fn();
    const mockedLoadChildren = jest.fn();
    const mockedMoveLinks = jest.fn();
    const mockedDeletePhotosShare = jest.fn();

    beforeEach(() => {
        jest.clearAllMocks();
        mockedDeletePhotosShare.mockResolvedValue(undefined);
        mockedLoadChildren.mockResolvedValue(undefined);

        mockedMoveLinks.mockImplementation(
            async (abortSignal: AbortSignal, { linkIds, onMoved }: { linkIds: string[]; onMoved?: () => void }) => {
                // Reproduce the async behavior of moveLinks
                linkIds.forEach(() => setTimeout(() => onMoved?.(), 10));
            }
        );

        // @ts-ignore
        mockedUseLinksListing.mockReturnValue({
            loadChildren: mockedLoadChildren,
            getCachedChildren: mockedGetCachedChildren,
        });
        // @ts-ignore
        mockedUsePhotos.mockReturnValue({
            shareId: 'shareId',
            linkId: 'linkId',
            deletePhotosShare: mockedDeletePhotosShare,
        });
        // @ts-ignore
        mockedUseLinksActions.mockReturnValue({
            moveLinks: mockedMoveLinks,
        });
        // @ts-ignore
        mockedUseShareState.mockReturnValue({
            getRestoredPhotosShares: () => [
                {
                    addressId: 'addressId',
                    shareId: 'shareId',
                    rootLinkId: 'rootLinkId',
                    volumeId: 'volumeId',
                    creator: 'creator',
                    isLocked: false,
                    isDefault: false,
                    isVolumeSoftDeleted: false,
                    possibleKeyPackets: ['dsad'],
                    type: 4,
                    state: 1,
                },
            ],
        });
    });

    it('should pass all state if files need to be recovered', async () => {
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Decrypting step
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Preparing step
        mockedGetCachedChildren.mockReturnValueOnce({ links: [], isDecrypting: false }); // Deleting step
        const { result } = renderHook(() => usePhotosRecovery());
        act(() => {
            result.current.start();
        });

        await waitFor(() => expect(result.current.state).toEqual('SUCCEED'));
        expect(result.current.countOfUnrecoveredLinksLeft).toEqual(0);
        expect(mockedGetCachedChildren).toHaveBeenCalledTimes(3);
        expect(mockedMoveLinks).toHaveBeenCalledTimes(1);
        expect(mockedLoadChildren).toHaveBeenCalledTimes(1);
        expect(mockedDeletePhotosShare).toHaveBeenCalledTimes(1);
        expect(result.current.countOfUnrecoveredLinksLeft).toEqual(0);
        expect(mockedRemoveItem).toHaveBeenCalledTimes(1);
        expect(mockedRemoveItem).toHaveBeenCalledWith('photos-recovery-state');
    });

    it('should pass and set errors count if some moves failed', async () => {
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Decrypting step
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Preparing step
        mockedGetCachedChildren.mockReturnValueOnce({ links: [links[0]], isDecrypting: false }); // Deleting step
        mockedMoveLinks.mockImplementation(
            async (
                abortSignal: AbortSignal,
                { linkIds, onMoved, onError }: { linkIds: string[]; onMoved?: () => void; onError?: () => void }
            ) => {
                linkIds.forEach((linkId) => {
                    if (linkId === 'linkId2') {
                        onError?.();
                    } else {
                        onMoved?.();
                    }
                });
            }
        );
        const { result } = renderHook(() => usePhotosRecovery());
        act(() => {
            result.current.start();
        });

        await waitFor(() => expect(result.current.state).toEqual('FAILED'));
        expect(result.current.countOfFailedLinks).toEqual(1);
        expect(result.current.countOfUnrecoveredLinksLeft).toEqual(0);
        expect(result.current.state).toEqual('FAILED');
        expect(mockedDeletePhotosShare).toHaveBeenCalledTimes(0);
        expect(mockedGetItem).toHaveBeenCalledTimes(1);
        expect(mockedSetItem).toHaveBeenCalledTimes(2);
        expect(mockedSetItem).toHaveBeenCalledWith('photos-recovery-state', 'progress');
        expect(mockedSetItem).toHaveBeenCalledWith('photos-recovery-state', 'failed');
    });

    it('should failed if deleteShare failed', async () => {
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Decrypting step
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Preparing step
        mockedGetCachedChildren.mockReturnValueOnce({ links: [], isDecrypting: false }); // Deleting step
        mockedDeletePhotosShare.mockRejectedValue(undefined);
        const { result } = renderHook(() => usePhotosRecovery());
        act(() => {
            result.current.start();
        });

        await waitFor(() => expect(result.current.state).toEqual('FAILED'));
        expect(mockedDeletePhotosShare).toHaveBeenCalledTimes(1);

        expect(mockedGetItem).toHaveBeenCalledTimes(1);
        expect(mockedSetItem).toHaveBeenCalledTimes(2);
        expect(mockedSetItem).toHaveBeenCalledWith('photos-recovery-state', 'progress');
        expect(mockedSetItem).toHaveBeenCalledWith('photos-recovery-state', 'failed');
    });

    it('should failed if loadChildren failed', async () => {
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Decrypting step
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Preparing step
        mockedGetCachedChildren.mockReturnValueOnce({ links: [], isDecrypting: false }); // Deleting step
        mockedLoadChildren.mockRejectedValue(undefined);
        const { result } = renderHook(() => usePhotosRecovery());
        act(() => {
            result.current.start();
        });

        await waitFor(() => expect(result.current.state).toEqual('FAILED'));
        expect(mockedDeletePhotosShare).toHaveBeenCalledTimes(0);
        expect(mockedGetCachedChildren).toHaveBeenCalledTimes(0);

        expect(mockedGetItem).toHaveBeenCalledTimes(1);
        expect(mockedSetItem).toHaveBeenCalledTimes(2);
        expect(mockedSetItem).toHaveBeenCalledWith('photos-recovery-state', 'progress');
        expect(mockedSetItem).toHaveBeenCalledWith('photos-recovery-state', 'failed');
    });

    it('should failed if moveLinks helper failed', async () => {
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Decrypting step
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Preparing step
        mockedGetCachedChildren.mockReturnValueOnce({ links: [], isDecrypting: false }); // Deleting step
        mockedMoveLinks.mockRejectedValue(undefined);
        const { result } = renderHook(() => usePhotosRecovery());
        act(() => {
            result.current.start();
        });

        await waitFor(() => expect(result.current.state).toEqual('FAILED'));
        expect(mockedDeletePhotosShare).toHaveBeenCalledTimes(0);
        expect(mockedMoveLinks).toHaveBeenCalledTimes(1);
        expect(mockedGetCachedChildren).toHaveBeenCalledTimes(2);
        expect(mockedGetItem).toHaveBeenCalledTimes(1);
        expect(mockedSetItem).toHaveBeenCalledTimes(2);
        expect(mockedSetItem).toHaveBeenCalledWith('photos-recovery-state', 'progress');
        expect(mockedSetItem).toHaveBeenCalledWith('photos-recovery-state', 'failed');
    });

    it('should start the process if localStorage value was set to progress', async () => {
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Decrypting step
        mockedGetCachedChildren.mockReturnValueOnce({ links, isDecrypting: false }); // Preparing step
        mockedGetCachedChildren.mockReturnValueOnce({ links: [], isDecrypting: false }); // Deleting step
        mockedGetItem.mockReturnValueOnce('progress');
        const { result } = renderHook(() => usePhotosRecovery());

        await waitFor(() => expect(result.current.state).toEqual('SUCCEED'));
        expect(mockedGetItem).toHaveBeenCalledTimes(1);
    });

    it('should set state to failed if localStorage value was set to failed', async () => {
        mockedGetItem.mockReturnValueOnce('failed');
        const { result } = renderHook(() => usePhotosRecovery());

        await waitFor(() => expect(result.current.state).toEqual('FAILED'));
    });
});
