import { useMemo, useState } from 'react';

import { c } from 'ttag';

import { Button, Href } from '@proton/atoms';
import { Icon, useModalState } from '@proton/components';
import { getKnowledgeBaseUrl } from '@proton/shared/lib/helpers/url';
import noop from '@proton/utils/noop';

import { getContactEmail } from '../../../helpers/message/messageRecipients';
import { useContactsMap } from '../../../hooks/contact/useContacts';
import { MessageVerification, MessageWithOptionalBody } from '../../../store/messages/messagesTypes';
import ContactResignModal from '../modals/ContactResignModal';

interface Props {
    message: MessageWithOptionalBody;
    messageVerification: MessageVerification | undefined;
    onResignContact: () => void;
}

const ExtraAskResign = ({ message, messageVerification, onResignContact }: Props) => {
    const { pinnedKeysVerified, senderPinnedKeys } = messageVerification || {};
    const { Address = '' } = message.Sender || {};
    const contactsMap = useContactsMap();

    const [contactID, setContactID] = useState<string>('');

    const [contactResignModalProps, setContactResignModalOpen, render] = useModalState();

    const contactEmail = useMemo(() => getContactEmail(contactsMap, Address), [contactsMap, Address]);

    const senderName = message?.Sender?.Name || message?.Sender?.Address; // No optional in translations

    if (pinnedKeysVerified || !senderPinnedKeys?.length) {
        return null;
    }

    const handleClick = () => {
        if (!contactEmail) {
            return;
        }

        setContactID(contactEmail.ContactID);

        setContactResignModalOpen(true);
    };

    return (
        <div
            className="bg-norm rounded border pr-2 md:pr-1 pb-2 md:pb-1 pt-1 pl-2 mb-3 flex flex-nowrap flex-column md:flex-row"
            data-testid="extra-ask-resign:banner"
        >
            <div className="md:flex-1 flex flex-nowrap mb-2 md:mb-0">
                <Icon name="exclamation-circle-filled" className="shrink-0 ml-0.5 mt-1 color-danger" />
                <span className="px-2 mt-0 flex-1 self-center">
                    <span className="mr-1">{c('Info').t`We could not verify the sender's trusted keys.`}</span>
                    <Href className="mr-4 inline-block" href={getKnowledgeBaseUrl('/address-verification')}>
                        {c('Info').t`Learn more`}
                    </Href>
                </span>
            </div>
            <span className="shrink-0 items-start flex w-full md:w-auto">
                <Button
                    size="small"
                    color="weak"
                    shape="outline"
                    fullWidth
                    className="rounded-sm"
                    onClick={handleClick}
                    data-testid="ask-resign-banner:verify-button"
                >{c('Action').t`Verify`}</Button>
            </span>

            {render && (
                <ContactResignModal
                    title={c('Title').t`Trust pinned keys?`}
                    submit={c('Action').t`Trust`}
                    onResign={onResignContact}
                    contacts={[{ contactID }]}
                    onResolve={noop}
                    onReject={noop}
                    {...contactResignModalProps}
                >
                    {c('Info')
                        .t`When you enabled trusted keys for ${senderName}, the public keys were added to the contact details.`}
                    <br />
                    {c('Info')
                        .t`There has been an error with the signature used to verify the contact details, which may be the result of a password reset.`}
                </ContactResignModal>
            )}
        </div>
    );
};

export default ExtraAskResign;
