import { ComponentPropsWithoutRef, useState } from 'react';

import { generateUID } from '@proton/components';
import { CALENDAR_APP_NAME } from '@proton/shared/lib/constants';
import clsx from '@proton/utils/clsx';

import { LogoProps } from './Logo';

type Props = ComponentPropsWithoutRef<'svg'> & Pick<LogoProps, 'variant' | 'size' | 'hasTitle'>;

const CalendarLogo = ({ variant = 'with-wordmark', size, className, hasTitle = true, ...rest }: Props) => {
    // This logo can be several times in the view, ids has to be different each time
    const [uid] = useState(generateUID('logo'));

    let logoWidth: number;

    switch (variant) {
        case 'glyph-only':
            logoWidth = 36;
            break;
        case 'wordmark-only':
            logoWidth = 334;
            break;
        default:
            logoWidth = 178;
            break;
    }

    return (
        <svg
            xmlns="http://www.w3.org/2000/svg"
            xmlnsXlink="http://www.w3.org/1999/xlink"
            viewBox={`0 0 ${logoWidth} 36`}
            width={logoWidth}
            height="36"
            fill="none"
            role="img"
            className={clsx('logo', size && variant === 'glyph-only' && `icon-size-${size}`, variant, className)}
            aria-labelledby={`${uid}-title`}
            {...rest}
        >
            {hasTitle && <title id={`${uid}-title`}>{CALENDAR_APP_NAME}</title>}
            {variant === 'glyph-only' && (
                <>
                    <path fill={`url(#${uid}-a)`} d="M7 6a3 3 0 0 0-3 3v1l1 1v17l20-10 1 1h6V9a3 3 0 0 0-3-3H7Z" />
                    <path
                        fill={`url(#${uid}-b)`}
                        d="M23.643 10H4v16.119A2.88 2.88 0 0 0 6.881 29H17l5.251-7.877A3.411 3.411 0 0 1 25.41 19H26v-6.643A2.357 2.357 0 0 0 23.643 10Z"
                    />
                    <path
                        fill="#B8D7FF"
                        d="M17 29v-1.075a2.5 2.5 0 0 1 .602-1.627l5.053-5.897a3.43 3.43 0 0 1 .345-.404l-.345.404A3.394 3.394 0 0 0 22 22.41V29h-5Z"
                    />
                    <path
                        fill="#8F69FF"
                        d="M24.6 25.386h1.02a.787.787 0 0 0 .802.596c.462-.001.766-.253.766-.623s-.316-.575-.943-.575h-.405v-.833h.352c.606 0 .854-.223.854-.555 0-.331-.269-.56-.642-.56a.642.642 0 0 0-.455.153.631.631 0 0 0-.219.425h-.985c.035-.58.497-1.414 1.654-1.414.928 0 1.57.525 1.57 1.264a1.1 1.1 0 0 1-.216.66c-.142.191-.34.335-.567.405v.015a1.117 1.117 0 0 1 .922 1.135c0 .801-.74 1.33-1.695 1.33-1.014.005-1.74-.567-1.813-1.424Zm5.292-3.306h.743v4.661h-.959v-3.576l-.923.622v-.942l1.14-.765Z"
                    />
                    <defs>
                        <radialGradient
                            id={`${uid}-b`}
                            cx="0"
                            cy="0"
                            r="1"
                            gradientTransform="matrix(44.9183 0 0 40.1408 25.159 -5.851)"
                            gradientUnits="userSpaceOnUse"
                        >
                            <stop offset=".503" stopColor="#6D4AFF" />
                            <stop offset=".994" stopColor="#06B8FF" />
                        </radialGradient>
                        <linearGradient
                            id={`${uid}-a`}
                            x1="27.523"
                            x2="11.191"
                            y1="21.036"
                            y2="-2.692"
                            gradientUnits="userSpaceOnUse"
                        >
                            <stop stopColor="#BFE8FF" />
                            <stop offset=".307" stopColor="#BFABFF" />
                            <stop offset="1" stopColor="#7341FF" />
                        </linearGradient>
                    </defs>
                </>
            )}

            {variant === 'with-wordmark' && (
                <>
                    <path fill={`url(#${uid}-a)`} d="M3 6a3 3 0 0 0-3 3v1l1 1v17l20-10 1 1h6V9a3 3 0 0 0-3-3H3Z" />
                    <path
                        fill={`url(#${uid}-b)`}
                        d="M19.643 10H0v16.119A2.88 2.88 0 0 0 2.881 29H13l5.251-7.877A3.411 3.411 0 0 1 21.41 19H22v-6.643A2.357 2.357 0 0 0 19.643 10Z"
                    />
                    <path
                        fill="#B8D7FF"
                        d="M13 29v-1.075a2.5 2.5 0 0 1 .602-1.627l5.053-5.897a3.42 3.42 0 0 1 .345-.403l-.345.403A3.394 3.394 0 0 0 18 22.41V29h-5Z"
                    />
                    <path
                        fill="#8F69FF"
                        d="M20.6 25.386h1.02a.787.787 0 0 0 .802.596c.462-.001.766-.253.766-.623s-.316-.575-.943-.575h-.405v-.833h.352c.606 0 .854-.223.854-.555 0-.331-.269-.56-.642-.56a.642.642 0 0 0-.455.153.631.631 0 0 0-.219.425h-.985c.035-.58.497-1.414 1.654-1.414.928 0 1.57.525 1.57 1.264a1.1 1.1 0 0 1-.216.66c-.142.191-.34.335-.567.405v.015a1.117 1.117 0 0 1 .922 1.135c0 .801-.74 1.33-1.695 1.33-1.014.005-1.74-.567-1.813-1.424Zm5.292-3.306h.743v4.661h-.959v-3.576l-.923.622v-.942l1.14-.765Z"
                    />
                    <path
                        fill="var(--logo-text-proton-color)"
                        d="M38 21.26v3.664h2.56V21.42a1.282 1.282 0 0 1 1.279-1.286h2.624a4.593 4.593 0 0 0 3.261-1.36 4.652 4.652 0 0 0 1.351-3.28c0-1.23-.486-2.41-1.35-3.282a4.603 4.603 0 0 0-3.265-1.358H38v4.58h2.56v-2.159h3.73c.58 0 1.134.232 1.544.644a2.2 2.2 0 0 1 0 3.105 2.18 2.18 0 0 1-1.544.643h-2.71a3.55 3.55 0 0 0-2.528 1.055 3.651 3.651 0 0 0-.776 1.166A3.54 3.54 0 0 0 38 21.26Zm11.473 3.664v-5.583c0-2.279 1.322-4.091 3.97-4.091a5.09 5.09 0 0 1 1.262.14v2.296c-.301-.02-.56-.02-.682-.02-1.402 0-2.005.646-2.005 1.955v5.303h-2.545Zm5.994-4.734c0-2.802 2.104-4.937 5.033-4.937 2.929 0 5.033 2.135 5.033 4.937 0 2.802-2.104 4.957-5.033 4.957-2.929 0-5.033-2.158-5.033-4.957Zm7.558 0c0-1.593-1.064-2.722-2.525-2.722-1.465 0-2.525 1.127-2.525 2.722 0 1.612 1.063 2.721 2.525 2.721 1.461 0 2.525-1.112 2.525-2.721Zm10.643 0c0-2.802 2.104-4.937 5.032-4.937 2.926 0 5.03 2.135 5.03 4.937 0 2.802-2.104 4.957-5.03 4.957-2.928 0-5.032-2.158-5.032-4.957Zm7.557 0c0-1.593-1.063-2.722-2.524-2.722-1.462 0-2.525 1.127-2.525 2.722 0 1.612 1.063 2.721 2.525 2.721 1.461 0 2.524-1.112 2.524-2.721Zm3.828 4.734v-5.38c0-2.499 1.583-4.294 4.41-4.294 2.806 0 4.39 1.793 4.39 4.294v5.38h-2.525v-5.18c0-1.39-.623-2.258-1.865-2.258-1.243 0-1.865.866-1.865 2.258v5.18h-2.545Zm-12.147-7.436h-2.747v3.528c0 1.23.44 1.793 1.703 1.793.12 0 .42 0 .802-.02v2.075c-.52.14-.981.223-1.484.223-2.124 0-3.569-1.29-3.569-3.728v-3.87h-1.706v-2.036h.424c.168 0 .335-.034.489-.097a1.284 1.284 0 0 0 .694-.698 1.28 1.28 0 0 0 .096-.492v-1.918h2.545v3.205H72.9v2.035h.006Z"
                    />
                    <path
                        fill="var(--logo-text-product-color)"
                        d="M102.372 24.208a6.762 6.762 0 0 1-2.562-2.567 7.615 7.615 0 0 1 0-7.285 6.762 6.762 0 0 1 2.562-2.567 7.225 7.225 0 0 1 3.654-.935 7.446 7.446 0 0 1 3.03.614c.874.375 1.646.95 2.255 1.676a5.903 5.903 0 0 1 1.227 2.499h-2.695a3.808 3.808 0 0 0-1.433-1.664 4.657 4.657 0 0 0-4.642-.02 4.381 4.381 0 0 0-1.611 1.672 5.07 5.07 0 0 0 0 4.752c.376.692.936 1.27 1.62 1.67.684.4 1.462.603 2.255.594a4.273 4.273 0 0 0 2.378-.626 3.83 3.83 0 0 0 1.433-1.667h2.695a5.932 5.932 0 0 1-1.233 2.516 5.989 5.989 0 0 1-2.249 1.679 7.404 7.404 0 0 1-3.03.597 7.171 7.171 0 0 1-3.654-.938Zm18.438-8.428a4.4 4.4 0 0 1 1.804 1.744c.448.806.672 1.718.652 2.642v4.752h-2.289l-.164-1.427a3.195 3.195 0 0 1-1.264 1.22 3.857 3.857 0 0 1-1.835.427 4.438 4.438 0 0 1-2.318-.626 4.48 4.48 0 0 1-1.661-1.767 5.387 5.387 0 0 1-.611-2.602 4.953 4.953 0 0 1 .663-2.539 4.734 4.734 0 0 1 1.824-1.787 5.217 5.217 0 0 1 2.6-.634 5.437 5.437 0 0 1 2.599.597Zm-.839 6.396c.497-.475.744-1.15.744-2.03a2.758 2.758 0 0 0-.715-1.979 2.346 2.346 0 0 0-1.792-.76 2.35 2.35 0 0 0-.988.192 2.376 2.376 0 0 0-.822.58 3.079 3.079 0 0 0-.721 1.976c0 .72.256 1.42.721 1.975a2.385 2.385 0 0 0 1.813.752 2.4 2.4 0 0 0 .95-.177 2.53 2.53 0 0 0 .81-.53Zm7.561 2.742h-2.58V11.066h2.58v13.852Zm11.038-3.9h-7.244a2.57 2.57 0 0 0 .879 1.475 2.498 2.498 0 0 0 1.591.52c.46.018.92-.063 1.348-.234.339-.154.617-.415.795-.74h2.39a3.93 3.93 0 0 1-1.672 2.304 5.04 5.04 0 0 1-2.795.8c-.907.02-1.806-.2-2.599-.637a4.45 4.45 0 0 1-1.755-1.776 5.268 5.268 0 0 1-.621-2.567 5.21 5.21 0 0 1 .63-2.567 4.552 4.552 0 0 1 1.755-1.776 4.981 4.981 0 0 1 2.53-.635 4.795 4.795 0 0 1 2.47.65 4.59 4.59 0 0 1 1.718 1.786c.419.78.635 1.653.623 2.54a9.17 9.17 0 0 1-.043.857Zm-6.385-3.188c-.439.357-.75.852-.879 1.404h4.869a2.152 2.152 0 0 0-.787-1.384 2.42 2.42 0 0 0-1.603-.546 2.478 2.478 0 0 0-1.6.526Zm7.365 1.664a4.297 4.297 0 0 1 .273-1.69c.204-.54.517-1.032.919-1.446.796-.787 1.893-1.178 3.292-1.178 1.398 0 2.493.4 3.286 1.198a4.287 4.287 0 0 1 1.183 3.139v5.418h-2.579v-5.238a2.599 2.599 0 0 0-.46-1.664 1.97 1.97 0 0 0-2.835 0 2.618 2.618 0 0 0-.483 1.673v5.22h-2.579l-.017-5.432Zm17.375-8.428h2.579v13.852h-2.346l-.181-1.47a3.09 3.09 0 0 1-1.235 1.24 3.92 3.92 0 0 1-1.908.447 4.23 4.23 0 0 1-2.134-.569 4.254 4.254 0 0 1-1.631-1.71 5.508 5.508 0 0 1-.624-2.699 5.475 5.475 0 0 1 .624-2.69 4.254 4.254 0 0 1 1.631-1.71 4.239 4.239 0 0 1 2.134-.569 4.01 4.01 0 0 1 1.856.41c.517.248.948.648 1.235 1.146v-5.678Zm-.715 11.101a2.726 2.726 0 0 0 .712-2.021c0-.866-.241-1.533-.721-1.996a2.406 2.406 0 0 0-.807-.529 2.455 2.455 0 0 0-.951-.18 2.46 2.46 0 0 0-.948.198 2.462 2.462 0 0 0-.798.543 2.723 2.723 0 0 0-.713 1.978c-.037.732.219 1.45.713 1.996a2.394 2.394 0 0 0 1.795.74 2.348 2.348 0 0 0 1.718-.729Zm12.222-6.387a4.41 4.41 0 0 1 1.804 1.744c.448.806.672 1.718.652 2.642v4.752h-2.293l-.163-1.427a3.243 3.243 0 0 1-1.267 1.22 3.857 3.857 0 0 1-1.835.427 4.423 4.423 0 0 1-3.979-2.393 5.4 5.4 0 0 1-.612-2.602 4.964 4.964 0 0 1 .664-2.539 4.76 4.76 0 0 1 1.821-1.784 5.242 5.242 0 0 1 2.605-.637 5.445 5.445 0 0 1 2.603.597Zm-.836 6.396c.497-.475.744-1.15.744-2.03a2.777 2.777 0 0 0-.712-1.979 2.508 2.508 0 0 0-2.781-.566c-.31.132-.589.323-.821.566a3.072 3.072 0 0 0 0 3.952 2.365 2.365 0 0 0 1.803.76 2.474 2.474 0 0 0 1.767-.703Zm4.579-2.886a4.23 4.23 0 0 1 1.051-2.97c.698-.76 1.695-1.14 2.987-1.14.434-.004.865.045 1.287.142v2.316c-.204 0-.437-.02-.692-.02-.707 0-1.224.163-1.551.489a2.065 2.065 0 0 0-.397.692 2.052 2.052 0 0 0-.095.789v5.34h-2.579l-.011-5.637Z"
                    />
                    <defs>
                        <radialGradient
                            id={`${uid}-b`}
                            cx="0"
                            cy="0"
                            r="1"
                            gradientTransform="matrix(44.9183 0 0 40.1408 21.159 -5.851)"
                            gradientUnits="userSpaceOnUse"
                        >
                            <stop offset=".503" stopColor="#6D4AFF" />
                            <stop offset=".994" stopColor="#06B8FF" />
                        </radialGradient>
                        <linearGradient
                            id={`${uid}-a`}
                            x1="23.523"
                            x2="7.191"
                            y1="21.036"
                            y2="-2.692"
                            gradientUnits="userSpaceOnUse"
                        >
                            <stop stopColor="#BFE8FF" />
                            <stop offset=".307" stopColor="#BFABFF" />
                            <stop offset="1" stopColor="#7341FF" />
                        </linearGradient>
                    </defs>
                </>
            )}

            {variant === 'wordmark-only' && (
                <>
                    <path
                        d="M322.431 20.9001C322.431 18.2426 323.18 16.1197 324.676 14.5313C326.204 12.943 328.251 12.1488 330.817 12.1488C331.58 12.1488 332.434 12.2557 333.383 12.4695V16.8681C333.108 16.8375 332.558 16.8223 331.733 16.8223C330.235 16.8223 329.105 17.2041 328.343 17.9677C327.61 18.7313 327.242 19.9837 327.242 21.7248V34.0956H322.431V20.9001Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M304.251 34.5538C301.441 34.5538 299.028 33.5458 297.012 31.5299C295.027 29.5139 294.034 26.7953 294.034 23.3743C294.034 19.9226 295.027 17.1888 297.012 15.1728C299.028 13.1568 301.441 12.1488 304.251 12.1488C305.84 12.1488 307.275 12.4848 308.558 13.1568C309.841 13.7983 310.849 14.7299 311.582 15.9517L312.04 12.607H316.21V34.0956H311.949L311.582 30.7968C310.055 33.3016 307.611 34.5538 304.251 34.5538ZM298.845 23.3743C298.845 25.4819 299.456 27.1619 300.678 28.4143C301.93 29.6361 303.457 30.247 305.259 30.247C306.97 30.247 308.451 29.6361 309.704 28.4143C310.956 27.1619 311.582 25.4819 311.582 23.3743C311.582 21.3277 310.941 19.6783 309.658 18.4259C308.405 17.143 306.939 16.5015 305.259 16.5015C303.457 16.5015 301.93 17.1277 300.678 18.3801C299.456 19.6325 298.845 21.2972 298.845 23.3743Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M284.652 34.0956L284.286 30.8426C282.82 33.3167 280.406 34.5538 277.047 34.5538C274.267 34.5538 271.884 33.5612 269.899 31.5757C267.913 29.5597 266.921 26.8259 266.921 23.3743C266.921 19.9226 267.913 17.1888 269.899 15.1728C271.884 13.1568 274.267 12.1488 277.047 12.1488C280.254 12.1488 282.621 13.3096 284.148 15.631V2.89355H289.005V34.0956H284.652ZM273.519 18.3801C272.327 19.6019 271.732 21.2666 271.732 23.3743C271.732 25.4819 272.327 27.1619 273.519 28.4143C274.71 29.6361 276.206 30.247 278.009 30.247C279.719 30.247 281.185 29.6361 282.407 28.4143C283.659 27.1619 284.286 25.4819 284.286 23.3743C284.286 21.2666 283.659 19.6019 282.407 18.3801C281.185 17.1277 279.719 16.5016 278.009 16.5016C276.206 16.5016 274.71 17.1277 273.519 18.3801Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M243.373 21.5873C243.373 18.7772 244.229 16.5015 245.939 14.7604C247.65 13.0193 249.986 12.1488 252.949 12.1488C255.912 12.1488 258.249 13.0193 259.96 14.7604C261.67 16.5015 262.525 18.7772 262.525 21.5873V34.0956H257.715V21.9539C257.715 20.2433 257.302 18.9146 256.477 17.9677C255.683 16.9902 254.507 16.5015 252.949 16.5015C251.392 16.5015 250.2 16.9902 249.376 17.9677C248.581 18.9146 248.184 20.2433 248.184 21.9539V34.0956H243.373V21.5873Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M228.567 34.5538C225.359 34.5538 222.732 33.5153 220.686 31.4383C218.639 29.3612 217.616 26.6732 217.616 23.3743C217.616 20.1059 218.624 17.4179 220.64 15.3103C222.687 13.2026 225.314 12.1488 228.521 12.1488C231.453 12.1488 233.927 13.2179 235.943 15.3561C237.959 17.4942 238.967 20.167 238.967 23.3743C238.967 24.0768 238.952 24.6572 238.922 25.1153H222.427C222.732 26.7648 223.435 28.0477 224.535 28.9641C225.634 29.8804 226.963 30.3386 228.521 30.3386C231.056 30.3386 232.782 29.4375 233.698 27.6354H238.418C237.776 29.7736 236.554 31.4688 234.752 32.7212C232.95 33.9431 230.888 34.5538 228.567 34.5538ZM222.427 21.4499H234.202C234.049 20.0448 233.438 18.8535 232.37 17.8761C231.3 16.8681 230.017 16.3641 228.521 16.3641C226.963 16.3641 225.634 16.8223 224.535 17.7386C223.465 18.655 222.763 19.8921 222.427 21.4499Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path d="M207.557 34.0956V2.89355H212.368V34.0956H207.557Z" fill="var(--logo-text-proton-color)" />
                    <path
                        d="M188.507 34.5538C185.697 34.5538 183.284 33.5458 181.268 31.5299C179.282 29.5139 178.289 26.7953 178.289 23.3743C178.289 19.9226 179.282 17.1888 181.268 15.1728C183.284 13.1568 185.697 12.1488 188.507 12.1488C190.095 12.1488 191.531 12.4848 192.814 13.1568C194.097 13.7983 195.105 14.7299 195.838 15.9517L196.296 12.607H200.465V34.0956H196.204L195.838 30.7968C194.31 33.3016 191.867 34.5538 188.507 34.5538ZM183.1 23.3743C183.1 25.4819 183.711 27.1619 184.933 28.4143C186.185 29.6361 187.713 30.247 189.515 30.247C191.225 30.247 192.707 29.6361 193.959 28.4143C195.211 27.1619 195.838 25.4819 195.838 23.3743C195.838 21.3277 195.196 19.6783 193.913 18.4259C192.661 17.143 191.195 16.5015 189.515 16.5015C187.713 16.5015 186.185 17.1277 184.933 18.3801C183.711 19.6325 183.1 21.2972 183.1 23.3743Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M160.461 34.5538C155.971 34.5538 152.214 33.0267 149.19 29.9721C146.166 26.887 144.654 23.0688 144.654 18.5175C144.654 13.9663 146.151 10.1481 149.144 7.063C152.168 3.97789 155.941 2.43536 160.461 2.43536C163.974 2.43536 167.029 3.39754 169.625 5.32191C172.252 7.24627 173.962 9.76627 174.757 12.8819H169.625C168.861 11.1408 167.67 9.73573 166.051 8.66663C164.432 7.59753 162.569 7.063 160.461 7.063C157.376 7.063 154.795 8.16263 152.718 10.3619C150.671 12.5306 149.648 15.2492 149.648 18.5175C149.648 21.7553 150.671 24.4739 152.718 26.6732C154.795 28.8419 157.376 29.9263 160.461 29.9263C162.569 29.9263 164.432 29.3917 166.051 28.3226C167.67 27.2535 168.861 25.8485 169.625 24.1074H174.757C173.962 27.223 172.252 29.743 169.625 31.6674C167.029 33.5916 163.974 34.5538 160.461 34.5538Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M109.822 21.5873C109.822 18.7772 110.677 16.5015 112.388 14.7604C114.099 13.0193 116.435 12.1488 119.398 12.1488C122.361 12.1488 124.698 13.0193 126.408 14.7604C128.119 16.5015 128.974 18.7772 128.974 21.5873V34.0956H124.163V21.9539C124.163 20.2433 123.751 18.9146 122.926 17.9677C122.132 16.9902 120.956 16.5015 119.398 16.5015C117.84 16.5015 116.649 16.9902 115.824 17.9677C115.03 18.9146 114.633 20.2433 114.633 21.9539V34.0956H109.822V21.5873Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M94.1717 34.5538C90.9645 34.5538 88.2916 33.4849 86.1536 31.3466C84.0155 29.2085 82.9463 26.551 82.9463 23.3743C82.9463 20.1975 84.0155 17.5401 86.1536 15.4019C88.2916 13.2332 90.9645 12.1488 94.1717 12.1488C97.4095 12.1488 100.098 13.2332 102.236 15.4019C104.374 17.5401 105.443 20.1975 105.443 23.3743C105.443 26.551 104.374 29.2085 102.236 31.3466C100.098 33.4849 97.4095 34.5538 94.1717 34.5538ZM87.7572 23.3743C87.7572 25.3597 88.3528 27.0092 89.5441 28.3226C90.7658 29.6055 92.3086 30.247 94.1717 30.247C96.0349 30.247 97.5777 29.6055 98.7994 28.3226C100.021 27.0092 100.632 25.3597 100.632 23.3743C100.632 21.3583 100.021 19.7088 98.7994 18.4259C97.5777 17.1125 96.0349 16.4557 94.1717 16.4557C92.3086 16.4557 90.7658 17.1125 89.5441 18.4259C88.3528 19.7088 87.7572 21.3583 87.7572 23.3743Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M76.573 34.4163C74.3125 34.4163 72.4644 33.7444 71.029 32.4005C69.6237 31.0259 68.9213 29.1321 68.9213 26.719V16.5932H65.0268V12.607H68.8755V5.41354H73.7322V12.607H80.0093V16.5932H73.7322V26.1234C73.7322 27.559 74.053 28.5823 74.6944 29.1932C75.3663 29.7735 76.4355 30.0637 77.9017 30.0637C78.4208 30.0637 78.986 30.0484 79.597 30.0179V34.004C78.5278 34.2789 77.5198 34.4163 76.573 34.4163Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M51.713 34.5538C48.5057 34.5538 45.8329 33.4849 43.6948 31.3466C41.5568 29.2085 40.4875 26.551 40.4875 23.3743C40.4875 20.1975 41.5568 17.5401 43.6948 15.4019C45.8329 13.2332 48.5057 12.1488 51.713 12.1488C54.9507 12.1488 57.6389 13.2332 59.777 15.4019C61.9151 17.5401 62.9843 20.1975 62.9843 23.3743C62.9843 26.551 61.9151 29.2085 59.777 31.3466C57.6389 33.4849 54.9507 34.5538 51.713 34.5538ZM45.2985 23.3743C45.2985 25.3597 45.8941 27.0092 47.0854 28.3226C48.3071 29.6055 49.8498 30.247 51.713 30.247C53.5762 30.247 55.1189 29.6055 56.3406 28.3226C57.5624 27.0092 58.1734 25.3597 58.1734 23.3743C58.1734 21.3583 57.5624 19.7088 56.3406 18.4259C55.1189 17.1125 53.5762 16.4557 51.713 16.4557C49.8498 16.4557 48.3071 17.1125 47.0854 18.4259C45.8941 19.7088 45.2985 21.3583 45.2985 23.3743Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M26.5155 20.9001C26.5155 18.2426 27.2638 16.1197 28.7606 14.5313C30.2878 12.943 32.3344 12.1488 34.9004 12.1488C35.6639 12.1488 36.5191 12.2557 37.4662 12.4695V16.8681C37.1913 16.8375 36.6415 16.8223 35.8167 16.8223C34.3198 16.8223 33.1897 17.2041 32.426 17.9677C31.6929 18.7313 31.3264 19.9837 31.3264 21.7248V34.0956H26.5155V20.9001Z"
                        fill="var(--logo-text-proton-color)"
                    />
                    <path
                        d="M0 34.0956V2.89355H12.5084C15.3491 2.89355 17.7164 3.77938 19.6102 5.55101C21.5345 7.32263 22.4967 9.58301 22.4967 12.3321C22.4967 15.0812 21.5345 17.3568 19.6102 19.159C17.7164 20.9306 15.3491 21.8165 12.5084 21.8165H4.90255V34.0956H0ZM4.90255 17.2346H11.592C13.5164 17.2346 14.9825 16.7765 15.9905 15.8601C16.9985 14.9437 17.5025 13.7677 17.5025 12.3321C17.5025 10.8965 16.9985 9.73575 15.9905 8.84992C15.0131 7.93355 13.5775 7.47537 11.6836 7.47537H4.90255V17.2346Z"
                        fill="var(--logo-text-proton-color)"
                    />
                </>
            )}
        </svg>
    );
};

export default CalendarLogo;
