import { render } from '@testing-library/react';

import { getRenewalNoticeText } from './RenewalNotice';

const RenewalNotice = (...props: Parameters<typeof getRenewalNoticeText>) => {
    return <div>{getRenewalNoticeText(...props)}</div>;
};

describe('<RenewalNotice />', () => {
    beforeEach(() => {
        jest.clearAllMocks();
        jest.useFakeTimers();
    });

    afterEach(() => {
        jest.useRealTimers();
    });

    it('should render', () => {
        const { container } = render(
            <RenewalNotice
                renewCycle={12}
                isCustomBilling={false}
                isScheduledSubscription={false}
                subscription={undefined}
            />
        );
        expect(container).not.toBeEmptyDOMElement();
    });

    it('should display the correct renewal date', () => {
        const mockedDate = new Date(2023, 10, 1);
        jest.setSystemTime(mockedDate);

        const renewCycle = 12;
        const expectedDateString = '11/01/2024'; // because months are 0-indexed ¯\_(ツ)_/¯

        const { container } = render(
            <RenewalNotice
                renewCycle={renewCycle}
                isCustomBilling={false}
                isScheduledSubscription={false}
                subscription={undefined}
            />
        );
        expect(container).toHaveTextContent(
            `Subscription auto-renews every 12 months. Your next billing date is ${expectedDateString}.`
        );
    });

    it('should use period end date if custom billing is enabled', () => {
        const mockedDate = new Date(2023, 10, 1);
        jest.setSystemTime(mockedDate);

        const renewCycle = 12;
        const expectedDateString = '08/11/2025'; // because months are 0-indexed ¯\_(ツ)_/¯

        const { container } = render(
            <RenewalNotice
                renewCycle={renewCycle}
                isCustomBilling={true}
                isScheduledSubscription={false}
                subscription={
                    {
                        // the backend returns seconds, not milliseconds
                        PeriodEnd: +new Date(2025, 7, 11) / 1000,
                    } as any
                }
            />
        );
        expect(container).toHaveTextContent(
            `Subscription auto-renews every 12 months. Your next billing date is ${expectedDateString}.`
        );
    });

    it('should use the end of upcoming subscription period if scheduled subscription is enabled', () => {
        const mockedDate = new Date(2023, 10, 1);
        jest.setSystemTime(mockedDate);

        const renewCycle = 24; // the upcoming subscription takes another 24 months
        const { container } = render(
            <RenewalNotice
                renewCycle={renewCycle}
                isCustomBilling={false}
                isScheduledSubscription={true}
                subscription={
                    {
                        // the backend returns seconds, not milliseconds
                        PeriodEnd: +new Date(2024, 1, 3) / 1000, // the current subscription period ends on 02/03/2024 (3rd of February 2024)
                    } as any
                }
            />
        );

        const expectedDateString = '02/03/2026'; // and finally the renewal date is 02/03/2026 (3rd of February 2026)

        expect(container).toHaveTextContent(
            `Subscription auto-renews every 24 months. Your next billing date is ${expectedDateString}.`
        );
    });
});
